﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ConsoleApplication3
{
    internal class Program
    {
        private static void Main(string[] args)
        {
            // Need to call the longrunningtask within a Task (as shown below) - just like you would 'delegate' a method call.
            Task task = new Task(TaskWithoutBlocking);

            // invoke the Task (the delegate)
            task.Start();

            // wait for the task to complete
            task.Wait();

            Console.ReadLine();
        }

        /// <summary>
        /// Notice the 'async' required in the signature of this method
        /// </summary>
        private static async void TaskWithoutBlocking()
        {
            Task<string> t = LongRunningTask();
            
            Console.WriteLine("See - I came right back. No blocking");
            
            Console.WriteLine("A-Waiting result from long running task......"); 
            
            string resultFromLongRunningTask = await t;
           
            Console.WriteLine("Result (of long running task) = " + resultFromLongRunningTask);
        }

        /// <summary>
        /// Simulate long running task. To return a string, use Task<string>
        /// NOTE: For simulating an async wait, we need to invoke something that returns an awaitable type (a Func is an awaitable type)             
        /// </summary>
        private static async Task<string> LongRunningTask()
        {
            Console.WriteLine("Entered long running task");

            // Thread.Sleep(5000) cannot be used - since it is synchronous and does not return an awaitable type;

            // We need an awaitable type here, func is an acceptable awaitable type 
            var func = Task<string>.Factory.StartNew(() => DoLongRunningSynchronousWork());            
            string retValue = await func;
            return retValue;
        }
         
        /// Simulate a long running task
        /// For an infinite wait - try Thread.Sleep(Timeout.Infinite)          
        private static string DoLongRunningSynchronousWork()
        {
            Stopwatch sw = new Stopwatch();
            sw.Start();
            Console.WriteLine("Starting long running task...");
            Console.WriteLine("This long running SYNCHRONOUS method was called asynchronously");
           

            Thread.Sleep(5000); // this is the long running task
            Console.WriteLine("Finished long running task...");
            sw.Stop();
            return "Total time of long running synchronous method = " + sw.ElapsedMilliseconds+ " milli seconds";
        }
    }
}
